@@ -4,8 +4,11 @@
 #   Common Unittest Routines for CJK codecs
 #
 
-import sys, codecs
-import unittest, re
+import codecs
+import os
+import re
+import sys
+import unittest
 from test import support
 from io import BytesIO
 
@@ -277,9 +280,9 @@ class TestBase_Mapping(unittest.TestCase):
     def __init__(self, *args, **kw):
         unittest.TestCase.__init__(self, *args, **kw)
         try:
-            self.open_mapping_file() # test it to report the error early
+            self.open_mapping_file().close() # test it to report the error early
         except IOError:
-            raise support.TestSkipped("Could not retrieve "+self.mapfileurl)
+            self.skipTest("Could not retrieve "+self.mapfileurl)
 
     def open_mapping_file(self):
         return support.open_urlresource(self.mapfileurl)
@@ -294,36 +297,38 @@ class TestBase_Mapping(unittest.TestCase):
         unichrs = lambda s: ''.join(map(chr, map(eval, s.split('+'))))
         urt_wa = {}
 
-        for line in self.open_mapping_file():
-            if not line:
-                break
-            data = line.split('#')[0].strip().split()
-            if len(data) != 2:
-                continue
+        with self.open_mapping_file() as f:
+            for line in f:
+                if not line:
+                    break
+                data = line.split('#')[0].strip().split()
+                if len(data) != 2:
+                    continue
 
-            csetval = eval(data[0])
-            if csetval <= 0x7F:
-                csetch = bytes([csetval & 0xff])
-            elif csetval >= 0x1000000:
-                csetch = bytes([(csetval >> 24), ((csetval >> 16) & 0xff),
-                                ((csetval >> 8) & 0xff), (csetval & 0xff)])
-            elif csetval >= 0x10000:
-                csetch = bytes([(csetval >> 16), ((csetval >> 8) & 0xff),
-                                (csetval & 0xff)])
-            elif csetval >= 0x100:
-                csetch = bytes([(csetval >> 8), (csetval & 0xff)])
-            else:
-                continue
+                csetval = eval(data[0])
+                if csetval <= 0x7F:
+                    csetch = bytes([csetval & 0xff])
+                elif csetval >= 0x1000000:
+                    csetch = bytes([(csetval >> 24), ((csetval >> 16) & 0xff),
+                                    ((csetval >> 8) & 0xff), (csetval & 0xff)])
+                elif csetval >= 0x10000:
+                    csetch = bytes([(csetval >> 16), ((csetval >> 8) & 0xff),
+                                    (csetval & 0xff)])
+                elif csetval >= 0x100:
+                    csetch = bytes([(csetval >> 8), (csetval & 0xff)])
+                else:
+                    continue
 
-            unich = unichrs(data[1])
-            if ord(unich) == 0xfffd or unich in urt_wa:
-                continue
-            urt_wa[unich] = csetch
+                unich = unichrs(data[1])
+                if ord(unich) == 0xfffd or unich in urt_wa:
+                    continue
+                urt_wa[unich] = csetch
 
-            self._testpoint(csetch, unich)
+                self._testpoint(csetch, unich)
 
     def _test_mapping_file_ucm(self):
-        ucmdata = self.open_mapping_file().read()
+        with self.open_mapping_file() as f:
+            ucmdata = f.read()
         uc = re.findall('<a u="([A-F0-9]{4})" b="([0-9A-F ]+)"/>', ucmdata)
         for uni, coded in uc:
             unich = chr(int(uni, 16))
@@ -340,6 +345,10 @@ class TestBase_Mapping(unittest.TestCase):
         if (csetch, unich) not in self.pass_dectest:
             self.assertEqual(str(csetch, self.encoding), unich)
 
-def load_teststring(encoding):
-    from test import cjkencodings_test
-    return cjkencodings_test.teststring[encoding]
+def load_teststring(name):
+    dir = os.path.join(os.path.dirname(__file__), 'cjkencodings')
+    with open(os.path.join(dir, name + '.txt'), 'rb') as f:
+        encoded = f.read()
+    with open(os.path.join(dir, name + '-utf8.txt'), 'rb') as f:
+        utf8 = f.read()
+    return encoded, utf8
